import java.util.Vector;
import java.io.*;

public class CodeBlock {

	static class CodeLine {
		Instruction i;
		int param;
		CodeLine(Instruction _i, int _p) {
			i = _i;
			param = _p;
		}
		public String toString() {
			return i +" "+param;
		}
	}

	Vector<CodeLine> lines = new Vector<CodeLine>();

	CodeLine getCode(int i) {
		return lines.elementAt(i);
	}

	void append(CodeBlock next) {
		lines.addAll(next.lines);		
	}
	
	int getLength() {
		return lines.size();
	}

	void addInstruction(Instruction i, int param) {
		CodeLine c = new CodeLine(i, param);
		lines.add(c);
	}

	CodeBlock(Instruction i, int param) {
		addInstruction(i, param);
	}

	CodeBlock() {
	}

	void print() {
		int ip = 0;
		for (CodeLine cl: lines) {
			System.out.printf("%03d %s %s\n", ip, cl.i, cl.param);
			ip++;
		}
	}

	void write(DataOutputStream dos) throws IOException {
		dos.writeInt(lines.size());
		for (CodeLine cl: lines) {
			dos.writeByte(cl.i.getInstNumber());
			dos.writeInt(cl.param);
		}
	}

	void load(DataInputStream dis) throws IOException {
		int size = dis.readInt();
		for (int i =0; i<size; i++) {
			Instruction ins = Instruction.getInstruction(dis.readByte());  
			int param = dis.readInt();
			addInstruction(ins, param);
		}
	}

}
