import java.io.*;
import java.util.*;

class Interpreter {

	int variables[];
	Vector<String> strings;
	CodeBlock cb;
	Scanner sc;

	Interpreter() {
		sc = new Scanner(System.in);
	}

	void load(String filename) throws Exception  {
		FileInputStream fis = new FileInputStream(filename);
		DataInputStream dis = new DataInputStream(fis);

		/*prepare space for variable*/
		int varcount = dis.readInt();
		variables = new int[varcount];
		/*read the strings*/
		int stringcount = dis.readInt();
		strings = new Vector<String>();
		for (int i =0; i<stringcount; i++) {
			strings.add(dis.readUTF());
		}
		cb = new CodeBlock();
		cb.load(dis);
		fis.close();
	}

	void call(int function,  Stack<Integer> s) {
		int v, val;
		switch (function) {
		case Instruction.PRINTVAR:
			v = s.pop();
			System.out.print(v);
			break;
		case Instruction.PRINTSTR:
			v = s.pop();
			System.out.print(strings.elementAt(v));
			break;
		case Instruction.PRINTLINE:
			System.out.println();
			break;
		case Instruction.INPUT:
			v = sc.nextInt();
			s.push(v);
			break;
		}
	}

	void run() throws Exception {
		int ip, v1, v2;
		ip = 0;
		cb.print();
		Stack<Integer> s = new Stack<Integer>();
		while (ip<cb.getLength()) {
			CodeBlock.CodeLine cl = cb.getCode(ip);
			switch (cl.i) {
			case PUSHVAR:
				s.push(variables[cl.param]);
				break;
			case PUSH:
				s.push(cl.param);
				break;
			case POP:
				v1 = s.pop();
				variables[cl.param] = v1;
				break;
			case PUSHSTR:
				s.push(cl.param);
				break;
			case ADD:
				v1 = s.pop();
				v2 = s.pop();
				s.push(v1 + v2);
				break;
			case NEG:
				v1 = s.pop();
				s.push(-v1);
				break;				
			case SUB:
				v1 = s.pop();
				v2 = s.pop();
				s.push(v2 - v1);
				break;
			case MUL:				
				v1 = s.pop();
				v2 = s.pop();
				s.push(v2 * v1);
				break;
			case DIV:
				v1 = s.pop();
				v2 = s.pop();
				s.push(v2 / v1);
				break;
			case MOD:
				v1 = s.pop();
				v2 = s.pop();
				s.push(v2 % v1);
				break;
			case CMPLT:
				v1 = s.pop();
				v2 = s.pop();
				s.push((v2 < v1)?1:0);
				break;
			case CMPLE:
				v1 = s.pop();
				v2 = s.pop();
				s.push((v2 <= v1)?1:0);
				break;
			case CMPGT:
				v1 = s.pop();
				v2 = s.pop();
				s.push((v2 > v1)?1:0);
				break;
			case CMPGE:
				v1 = s.pop();
				v2 = s.pop();
				s.push((v2 >= v1)?1:0);
				break;
			case CMPEQ:
				v1 = s.pop();
				v2 = s.pop();
				s.push((v2 == v1)?1:0);
				break;
			case CMPNE:
				v1 = s.pop();
				v2 = s.pop();
				s.push((v2 != v1)?1:0);
				break;
			case JUMP: 
				ip = ip + cl.param;
				continue;
			case JUMPFALSE:
				v1 = s.pop();
				if (v1==0) {
					ip = ip + cl.param;
					continue;
				}
			case CALL:
				call (cl.param, s);
				break;
			default:
				throw new Exception("undefined instruction");
				

			}
			ip++;
		}
	}

	public static void main(String argv[]) throws Exception {
		if (argv.length!=1) {
			System.out.println("Usage: Interpreter <filename.bc>");
			return;
		}
		Interpreter i = new Interpreter();
		i.load(argv[0]);
		i.run();
	}
}
